# -*- coding: utf-8 -*-

import wx
import vision
from gui import guiHelper,nvdaControls
from config import conf
import gui
from logHandler import log
import api
import watchdog
import addonHandler
addonHandler.initTranslation()

class USASettingsPanel(gui.SettingsPanel):
	# Translators: This is the label for the USA settings dialog.
	title = "Ultimate Screen Access"

	def makeSettings(self, sizer):
		from .usa import USA
		sHelper = guiHelper.BoxSizerHelper(self, sizer=sizer)
		if isinstance(vision.handler.magnifier,USA):
			# Translators: This is a grouping label in the
			# settings dialog.
			magGroup = guiHelper.BoxSizerHelper(self, sizer=wx.StaticBoxSizer(wx.StaticBox(self, label=_("Magnification")), wx.VERTICAL))
			sHelper.addItem(magGroup)
			magnificationLevelLabelText=_("Level")
			self.magnificationLevelEdit=magGroup.addLabeledControl(magnificationLevelLabelText,nvdaControls.SelectOnFocusSpinCtrl,min=1,max=60,initial=conf['vision']['usa']['magnificationLevel'])
			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.bitmapSmoothingCheckbox=magGroup.addItem(wx.CheckBox(self,label=_("Enable &bitmap smoothing to increase magnification quality")))
			self.bitmapSmoothingCheckbox.SetValue(conf['vision']['usa']["bitmapSmoothing"])

			# Translators: This is a grouping label in the
			# settings dialog.
			trackGroup = guiHelper.BoxSizerHelper(self, sizer=wx.StaticBoxSizer(wx.StaticBox(self, label=_("Tracking")), wx.VERTICAL))
			sHelper.addItem(trackGroup)
			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.trackToFocusCheckBox=trackGroup.addItem(wx.CheckBox(self,label=_("Follow &focus")))
			self.trackToFocusCheckBox.SetValue(conf['vision']['usa']["trackToFocus"])

			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.trackToNavigatorObjCheckBox=trackGroup.addItem(wx.CheckBox(self,label=_("Follow &navigator object")))
			self.trackToNavigatorObjCheckBox.SetValue(conf['vision']['usa']["trackToNavigatorObj"])
			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.trackToCaretCheckBox=trackGroup.addItem(wx.CheckBox(self,label=_("Follow system &caret")))
			self.trackToCaretCheckBox.SetValue(conf['vision']['usa']["trackToCaret"])
			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.trackToReviewCheckBox=trackGroup.addItem(wx.CheckBox(self,label=_("Follow &review cursor")))
			self.trackToReviewCheckBox.SetValue(conf['vision']['usa']["trackToReview"])
			# Translators: This is the label for a checkbox in the
			# settings dialog.
			self.trackToMouseCheckBox=trackGroup.addItem(wx.CheckBox(self,label=_("Follow &mouse")))
			self.trackToMouseCheckBox.SetValue(conf['vision']['usa']["trackToMouse"])

		if isinstance(vision.handler.colorEnhancer,USA):
			# Translators: This is a grouping label in the
			# settings dialog.
			colorGroup = guiHelper.BoxSizerHelper(self, sizer=wx.StaticBoxSizer(wx.StaticBox(self, label=_("Color enhancements")), wx.VERTICAL))
			sHelper.addItem(colorGroup)

			# Translators: This is the label for a combobox in the
			# settings dialog.
			colorSchemeLabelText = "Color &scheme"
			self.colorSchemeValues = list(vision.handler.colorEnhancer.availableTransformations.keys())
			colorSchemeChoices = list(t.name for t in vision.handler.colorEnhancer.availableTransformations.values())
			self.colorSchemeList=colorGroup.addLabeledControl(colorSchemeLabelText, wx.Choice, choices=colorSchemeChoices)
			try:
				self.colorSchemeList.Selection=self.colorSchemeValues.index(conf['vision']['usa']['colorScheme'])
			except:
				log.debugWarning("Could not set color scheme list to current scheme",exc_info=True)

	def onSave(self):
		from .usa import USA
		conf['vision']['usa']["magnificationLevel"] = self.magnificationLevelEdit.GetValue()
		if isinstance(vision.handler.magnifier,USA):
			vision.handler.magnifier.magnificationLevel = conf['vision']['usa']["magnificationLevel"]
		conf['vision']['usa']["bitmapSmoothing"] = self.bitmapSmoothingCheckbox.IsChecked()
		if isinstance(vision.handler.magnifier,USA):
			vision.handler.magnifier.bitmapSmoothing = conf['vision']['usa']["bitmapSmoothing"] 
		conf['vision']['usa']["trackToFocus"]=self.trackToFocusCheckBox.IsChecked()
		conf['vision']['usa']["trackToNavigatorObj"]=self.trackToNavigatorObjCheckBox.IsChecked()
		conf['vision']['usa']["trackToCaret"]=self.trackToCaretCheckBox.IsChecked()
		conf['vision']['usa']["trackToReview"]=self.trackToReviewCheckBox.IsChecked()
		conf['vision']['usa']["trackToMouse"]=self.trackToMouseCheckBox.IsChecked()
		colorScheme=self.colorSchemeValues[self.colorSchemeList.Selection]
		if isinstance(vision.handler.colorEnhancer,USA):
			vision.handler.colorEnhancer.transformation = colorScheme
		else:
			conf['vision']['usa']["colorScheme"] = colorScheme

class USALicenseDialog(wx.Dialog):

	def __init__(self, parent, id, title):
		try:
			import libUSA
		except:
			wx.CallAfter(gui.messageBox,_("The USADVC library is not available on your system. Managing a license is therefore not supported."), _("Error"), wx.OK | wx.ICON_ERROR)
			return
		super(USALicenseDialog, self).__init__(parent, id, title=title)
		self.isLicensed = libUSA.IsLicensed()
		if self.isLicensed:
			message = _("Your copy of Ultimate Screen Access is properly licensed.\nChoose OK to deactivate the product, or Cancel to close this dialog.")
		else:
			message = _("Your copy of Ultimate Screen Access doesn't seem to be licensed.\nEnter your e-mail address and license key in the respective fields and choose OK to activate the product.\nAlternatively, press Cancel to close this dialog.")
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		main_sizer.Add(wx.StaticText(self, label=message))
		main_sizer_helper = gui.guiHelper.BoxSizerHelper(self, orientation=wx.VERTICAL)
		#Translators: The input field to enter an e-mail address
		self.emailAddress = main_sizer_helper.addLabeledControl(_("&E-mail Address:"), wx.TextCtrl)
		self.emailAddress.Enabled=not self.isLicensed
		#Translators: The input field to enter the USA license key
		self.key = main_sizer_helper.addLabeledControl(_("&License Key:"), wx.TextCtrl)
		self.key.Enabled=not self.isLicensed
		main_sizer_helper.addDialogDismissButtons(self.CreateButtonSizer(wx.OK | wx.CANCEL))
		self.Bind(wx.EVT_BUTTON, self.on_ok, id=wx.ID_OK)
		main_sizer.Add(main_sizer_helper.sizer, border = gui.guiHelper.BORDER_FOR_DIALOGS, flag=wx.ALL)
		main_sizer.Fit(self)
		self.SetSizer(main_sizer)
		self.Center(wx.BOTH | wx.CENTER_ON_SCREEN)
		self.Show()
		self.emailAddress.SetFocus()

	def on_ok(self, evt):
		if not self.isLicensed:
			self.activate(evt)
		else:
			self.deactivate(evt)

	def activate(self, evt):
		if not self.emailAddress.Value or not self.key.Value:
			gui.messageBox(_("You must enter a valid e-mail address and license key."), _("Error"), wx.OK | wx.ICON_ERROR)
			self.emailAddress.SetFocus()
			return
		progressDialog = gui.IndeterminateProgressDialog(self, _("Performing request"), _("Please wait while your license is being activated..."))

		try:
			succes, message = watchdog.cancellableExecute(libUSA.ActivateLicense, self.emailAddress.Value, self.key.Value)
		except:
			success = False
			message = _("There was an error while performing your request.")

		if not succes:
			wx.CallAfter(gui.messageBox,
				_("An error has occured:\n{error}").format(error=message),
					_("Error"), wx.OK | wx.ICON_ERROR)
		else:
			evt.Skip()
			wx.CallAfter(gui.messageBox,
				_("Ultimate Screen Access has been activated!\nAdditional info: {message}").format(message=message),
					_("Congratulations!"), wx.OK | wx.ICON_EXCLAMATION)
		progressDialog.done()

	def deactivate(self, evt):
		progressDialog = gui.IndeterminateProgressDialog(self, _("Performing request"), _("Please wait while your license is being deactivated..."))

		try:
			succes, message = watchdog.cancellableExecute(libUSA.DeactivateLicense)
		except:
			success = False
			message = _("There was a timeout while performing your request.")

		if not succes:
			wx.CallAfter(gui.messageBox,
				_("An error has occured:\n{error}").format(error=message),
					_("Error"), wx.OK | wx.ICON_ERROR)
		else:
			evt.Skip()
			wx.CallAfter(gui.messageBox,
				_("Ultimate Screen Access has been deactivated!\nAdditional info: {message}").format(message=message),
					_("Congratulations!"), wx.OK | wx.ICON_EXCLAMATION)
		progressDialog.done()
