from ctypes import *
from ctypes.wintypes import *
import os

class MAGCOLOREFFECT(Structure):
	_fields_ = (("transform", c_float*5*5),)

MagnifierType = c_int

magWindows = dict()

def errCheck(result, func, args):
	if result >0:
		raise WinError(result)
	return args

try:
	libUSA = windll.libUSA
except:
	libUSA = windll.LoadLibrary(os.path.join(os.path.abspath(os.path.dirname(__file__)), "libUSA.dll"))

# Prototype declarations
InitializeFuncType = WINFUNCTYPE(DWORD)
ActivateLicenseFuncType = WINFUNCTYPE(c_wchar_p, c_wchar_p, c_wchar_p, POINTER(BOOL))
DeactivateLicenseFuncType = WINFUNCTYPE(c_wchar_p, POINTER(BOOL))
TerminateFuncType = WINFUNCTYPE(DWORD)
IsLicensedFuncType = WINFUNCTYPE(BOOL)
CreateMagnifierFuncType = WINFUNCTYPE(HWND, RECT)
CreateMagnifierArgTypes = ((1, "rect"),)
DestroyMagnifierFuncType = WINFUNCTYPE(DWORD, HWND)
DestroyMagnifierArgTypes = ((1, "hwnd"),)
SetTrackingRectangleFuncType = WINFUNCTYPE(DWORD, HWND, RECT, RECT)
SetTrackingRectangleArgTypes = ((1, "hwnd"), (1, "rect"), (1, "boundary", RECT(-1,-1,-1,-1)),)
GetTrackingRectangleFuncType = WINFUNCTYPE(DWORD, HWND, POINTER(RECT))
GetTrackingRectangleArgTypes = ((1, "hwnd"), (2, "rect"),)
SetMagnificationLevelFuncType = WINFUNCTYPE(DWORD, HWND, c_float)
SetMagnificationLevelArgTypes = ((1, "hwnd"), (1, "magLevel"),)
GetMagnificationLevelFuncType = WINFUNCTYPE(DWORD, HWND, POINTER(c_float))
GetMagnificationLevelArgTypes = ((1, "hwnd"),(2, "magLevel"),)
SetColorSchemeFuncType = WINFUNCTYPE(DWORD, HWND, MAGCOLOREFFECT)
SetColorSchemeArgTypes = ((1, "hwnd"), (1, "colorScheme"),)
GetColorSchemeFuncType = WINFUNCTYPE(DWORD, HWND, POINTER(MAGCOLOREFFECT))
GetColorSchemeArgTypes = ((1, "hwnd"), (2, "colorScheme"),)
ShowSystemCursorFuncType = WINFUNCTYPE(DWORD, BOOL)
ShowSystemCursorArgTypes = ((1, "showCursor"),)
SetBitmapSmoothingFuncType = WINFUNCTYPE(DWORD, HWND, BOOL)
SetBitmapSmoothingArgTypes = ((1, "hwnd"), (1, "useSmoothing"),)

Initialize = InitializeFuncType(("USA_Initialize", libUSA))
Initialize.errcheck = errCheck
_ActivateLicense=ActivateLicenseFuncType(('USA_ActivateLicense', libUSA),((1,'emailAddress'),(1,'licenseKey'),(1,'success')))
_DeactivateLicense = DeactivateLicenseFuncType(('USA_DeactivateLicense', libUSA),((1,'success'),))
Terminate = TerminateFuncType(("USA_Terminate", libUSA))
Terminate.errcheck = errCheck
_CreateMagnifier = CreateMagnifierFuncType(("USA_CreateMagnifier", libUSA), CreateMagnifierArgTypes)
_DestroyMagnifier = DestroyMagnifierFuncType(("USA_DestroyMagnifier", libUSA), DestroyMagnifierArgTypes)
_DestroyMagnifier.errcheck =errCheck
SetTrackingRectangle = SetTrackingRectangleFuncType(("USA_SetTrackingRectangle", libUSA), SetTrackingRectangleArgTypes)
SetTrackingRectangle.errcheck = errCheck
GetTrackingRectangle = GetTrackingRectangleFuncType(("USA_GetTrackingRectangle", libUSA), GetTrackingRectangleArgTypes)
GetTrackingRectangle.errcheck = errCheck
SetMagnificationLevel = SetMagnificationLevelFuncType(("USA_SetMagnificationLevel", libUSA), SetMagnificationLevelArgTypes)
SetMagnificationLevel.errcheck = errCheck
GetMagnificationLevel = GetMagnificationLevelFuncType(("USA_GetMagnificationLevel", libUSA), GetMagnificationLevelArgTypes)
GetMagnificationLevel.errcheck = errCheck
SetColorScheme = SetColorSchemeFuncType(("USA_SetColorScheme", libUSA), SetColorSchemeArgTypes)
SetColorScheme.errcheck = errCheck
GetColorScheme = GetColorSchemeFuncType(("USA_GetColorScheme", libUSA), GetColorSchemeArgTypes)
GetColorScheme.errcheck = errCheck
ShowSystemCursor = ShowSystemCursorFuncType(("USA_ShowSystemCursor", libUSA), ShowSystemCursorArgTypes)
ShowSystemCursor.errcheck = errCheck
SetBitmapSmoothing = SetBitmapSmoothingFuncType(("USA_SetBitmapSmoothing", libUSA), SetBitmapSmoothingArgTypes)
SetBitmapSmoothing.errcheck = errCheck

def ActivateLicense(emailAddress, licenseKey):
	success = BOOL()
	message = _ActivateLicense(emailAddress, licenseKey, byref(success))
	return (success, message)

def DeactivateLicense():
	success = BOOL()
	message = _DeactivateLicense(byref(success))
	return (success, message)

def CreateMagnifier(rect):
	hwnd = _CreateMagnifier(rect)
	if hwnd:
		magWindows[hwnd] = rect
	return hwnd

def DestroyMagnifier(hwnd):
	try:
		del magWindows[hwnd]
	except KeyError:
		raise ValueError("hwnd not registered")
	return _DestroyMagnifier(hwnd)
